@@ -1,6 +1,8 @@
 #ifndef DATALAYOUT_H
 #define DATALAYOUT_H
 
+#include "abstractdatawidget.h"
+#include "dataformsbackend.h"
 #include <QWidget>
 #include <QGridLayout>
 #include <QHostAddress>
@@ -13,20 +15,23 @@ Q_DECLARE_METATYPE(QHostAddress);
 namespace Core
 {
 using namespace qutim_sdk_0_3;
+class DefaultDataForm;
 
 class DataLayout : public QGridLayout, public AbstractDataWidget
 {
-	Q_INTERFACES(qutim_sdk_0_3::AbstractDataWidget)
+	Q_INTERFACES(Core::AbstractDataWidget)
 public:
-	DataLayout(bool editable, QWidget *parent = 0);
+	DataLayout(const DataItem &item, DefaultDataForm *dataForm, quint8 columns, QWidget *parent = 0);
 	~DataLayout();
 	DataItem item() const;
+	DataItem item(bool hasSubitems) const;
 	void addItem(const DataItem &item);
 	void addItems(const QList<DataItem> &items);
 	void addSpacer();
 	void addRow(QWidget *widget) { addRow(0, widget); }
 	void addRow(QWidget *title, QWidget *widget, Qt::Alignment widgetAligment = 0);
 	bool isExpandable() { return m_expandable; }
+	void setHorizontalSpacing(int spacing);
 protected:
 	Qt::Alignment labelAlignment();
 private:
@@ -41,8 +46,11 @@ private:
 		QWidget *data;
 	};
 	QList<WidgetLine> m_widgets;
-	bool m_editable;
 	bool m_expandable;
+#ifndef QUTIM_MOBILE_UI
+	quint8 m_columns;
+	quint8 m_currentColumn;
+#endif
 };
 
 }
