@@ -49,7 +49,10 @@ QByteArray Channel1MessageData::fromUnicode(const QString &message, Channel1Code
 		codec = utf16Codec();
 	else
 		codec = asciiCodec();
-	return codec->fromUnicode(message);
+	QByteArray data = codec->fromUnicode(message);
+	if (charset == CodecUtf16Be)
+		data = data.mid(2); // Remove BOM which are shown by some clients as an unknown symbol
+	return data;
 }
 
 void Channel1MessageData::init(const QByteArray &message, Channel1Codec charset)
@@ -206,7 +209,6 @@ void MessagesHandler::handleSNAC(AbstractConnection *conn, const SNAC &sn)
 	}
 	// Server sends ICBM service parameters to client
 	case MessageFamily << 16 | MessageSrvReplyIcbm: {
-		debug() << IMPLEMENT_ME << "MessageFamily, MessageSrvReplyIcbm";
 		quint32 dw_flags = 0x00000303;
 		// TODO: Find description
 #ifdef DBG_CAPHTML
@@ -342,18 +344,23 @@ void MessagesHandler::handleMessage(IcqAccount *account, const SNAC &snac)
 	QString message;
 	switch (channel) {
 	case 0x0001: // message
-		message = handleChannel1Message(snac, contact, tlvs);
+		message = handleChannel1Message(contact, tlvs);
 		break;
 	case 0x0002: // rendezvous
-		message = handleChannel2Message(snac, contact, tlvs, cookie);
+		message = handleChannel2Message(contact, tlvs, cookie);
 		break;
 	case 0x0004:
-		message = handleChannel4Message(snac, contact, tlvs);
+		message = handleChannel4Message(contact, tlvs);
 		break;
 	default:
 		qWarning("Unknown message channel: %d", int(channel));
 	}
 	if (!message.isEmpty()) {
+		// qip always requires a message response, even if it has sent
+		// us the message on the channel 1.
+		// TODO: maybe there is another SNAC for the message responses?
+		if (contact->d_func()->flags & srvrelay_support)
+			sendChannel2Response(contact, MsgPlain, 0, cookie);
 		Message m;
 		if (tlvs.contains(0x0016))
 			m.setTime(QDateTime::fromTime_t(tlvs.value(0x0016).read<quint32>()));
@@ -396,9 +403,8 @@ void MessagesHandler::handleResponse(IcqAccount *account, const SNAC &snac)
 	handleTlv2711(snac, contact, 2, cookie);
 }
 
-QString MessagesHandler::handleChannel1Message(const SNAC &snac, IcqContact *contact, const TLVMap &tlvs)
+QString MessagesHandler::handleChannel1Message(IcqContact *contact, const TLVMap &tlvs)
 {
-	Q_UNUSED(snac);
 	QString message;
 	if (tlvs.contains(0x0002)) {
 		DataUnit data(tlvs.value(0x0002));
@@ -428,7 +434,7 @@ QString MessagesHandler::handleChannel1Message(const SNAC &snac, IcqContact *con
 	return message;
 }
 
-QString MessagesHandler::handleChannel2Message(const SNAC &snac, IcqContact *contact, const TLVMap &tlvs, quint64 msgCookie)
+QString MessagesHandler::handleChannel2Message(IcqContact *contact, const TLVMap &tlvs, quint64 msgCookie)
 {
 	QString uin = contact->id();
 	if (tlvs.contains(0x0005)) {
@@ -479,9 +485,8 @@ QString MessagesHandler::handleChannel2Message(const SNAC &snac, IcqContact *con
 	return QString();
 }
 
-QString MessagesHandler::handleChannel4Message(const SNAC &snac, IcqContact *contact, const TLVMap &tlvs)
+QString MessagesHandler::handleChannel4Message(IcqContact *contact, const TLVMap &tlvs)
 {
-	Q_UNUSED(snac);
 	QString uin = contact->id();
 	// TODO: Understand this holy shit
 	if (tlvs.contains(0x0005)) {
@@ -530,7 +535,7 @@ QString MessagesHandler::handleTlv2711(const DataUnit &data, IcqContact *contact
 
 		if (type == MsgPlain && ack != 2) // Plain message
 		{
-			contact->account()->d_func()->messageSender->appendChannel2Response(contact, type, flags, msgCookie);
+			//sendChannel2Response(contact, type, flags, msgCookie); // That was moved to handleMessage().
 			QByteArray message_data = data.read<QByteArray, quint16>(LittleEndian);
 			message_data.resize(message_data.size() - 1);
 			QColor foreground(data.read<quint8>(),
@@ -610,6 +615,16 @@ void MessagesHandler::sendMetaInfoRequest(IcqAccount *account, quint16 type)
 	account->connection()->send(snac);
 }
 
+void MessagesHandler::sendChannel2Response(IcqContact *contact, quint8 type, quint8 flags, const Cookie &cookie)
+{
+	Tlv2711 responseTlv(type, flags, 0, 0);
+	responseTlv.appendEmptyPacket();
+	responseTlv.appendColors();
+	ServerResponseMessage response(contact, 2, 3, cookie);
+	response.append(responseTlv.data());
+	contact->account()->connection()->send(response);
+}
+
 MessagePlugin::~MessagePlugin()
 {
 }
@@ -623,8 +638,6 @@ MessageSender::MessageSender(IcqAccount *account) :
 {
 	m_messagesTimer.setInterval(500);
 	connect(&m_messagesTimer, SIGNAL(timeout()), SLOT(sendMessage()));
-	m_responseTimer.setInterval(2000);
-	connect(&m_responseTimer, SIGNAL(timeout()), SLOT(sendResponse()));
 }
 
 bool MessageSender::appendMessage(IcqContact *contact, const Message &message)
@@ -650,21 +663,6 @@ bool MessageSender::appendMessage(IcqContact *contact, const Message &message)
 	return true;
 }
 
-void MessageSender::appendChannel2Response(IcqContact *contact, quint8 type, quint8 flags, const Cookie &cookie)
-{
-	if (m_responses.isEmpty() &&
-		m_account->connection()->testRate(MessageFamily, MessageResponse, false))
-	{
-		Q_ASSERT(!m_responseTimer.isActive());
-		sendResponse(contact, type, flags, cookie);
-	} else {
-		Channel2Response responseData = { contact, type, flags, cookie };
-		m_responses.push_back(responseData);
-		if (!m_responseTimer.isActive())
-			m_responseTimer.start();
-	}
-}
-
 void MessageSender::sendMessage()
 {
 	QList<MessageData>::iterator itr = m_messages.begin();
@@ -677,17 +675,6 @@ void MessageSender::sendMessage()
 	}
 }
 
-void MessageSender::sendResponse()
-{
-	QList<Channel2Response>::iterator itr = m_responses.begin();
-	if (m_account->connection()->testRate(MessageFamily, MessageResponse, false)) {
-		sendResponse(itr->contact, itr->type, itr->flags, itr->cookie);
-		m_responses.takeFirst();
-		if (m_responses.isEmpty())
-			m_responseTimer.stop();
-	}
-}
-
 MessageSender::MessageData::MessageData(IcqContact *contact_, const Message &message_) :
 	contact(contact_), message(message_)
 {
@@ -789,30 +776,20 @@ void MessageSender::sendMessage(MessageData &message)
 	// with wrong encoding
 	QByteArray msg = message.msgs.takeFirst();
 	Cookie cookie(message.contact, message.msgs.isEmpty() ? message.id : Cookie::generateId());
+	SNAC msgData;
 	if (message.channel == 1) {
-		ServerMessage msgData(contact, Channel1MessageData(msg, CodecUtf16Be), cookie);
-		m_account->connection()->send(msgData, 80);
+		msgData = ServerMessage(contact, Channel1MessageData(msg, CodecUtf16Be), cookie);
 	} else {
 		Tlv2711 tlv(0x01, 0, d->status.subtype(), 1, cookie);
 		tlv.append<quint16>(msg, LittleEndian);
 		tlv.appendColors();
 		if (message.utfEnabled)
 			tlv.append<quint32>(ICQ_CAPABILITY_UTF8.toString().toUpper(), LittleEndian);
-		ServerMessage msgData(contact, Channel2MessageData(0, tlv));
-		if (message.msgs.isEmpty())
-			msgData.setCookie(cookie, this, SLOT(messageTimeout(Cookie)));
-		m_account->connection()->send(msgData, 80);
+		msgData = ServerMessage(contact, Channel2MessageData(0, tlv));
 	}
-}
-
-void MessageSender::sendResponse(IcqContact *contact, quint8 type, quint8 flags, const Cookie &cookie)
-{
-	Tlv2711 responseTlv(type, flags, 0, 0);
-	responseTlv.appendEmptyPacket();
-	responseTlv.appendColors();
-	ServerResponseMessage response(contact, 2, 3, cookie);
-	response.append(responseTlv.data());
-	m_account->connection()->send(response);
+	if (message.msgs.isEmpty() && (d->flags & srvrelay_support))
+		msgData.setCookie(cookie, this, "messageTimeout(Cookie)");
+	m_account->connection()->send(msgData);
 }
 
 void MessageSender::messageTimeout(const Cookie &cookie)
